* PRISMOD MODEL - dopris0_master.do *
* Libor Dusek, Jan Vavra, Paolo Buonanno
** more info in dopris0_readme.do

*** PRISON MODEL - A GENERAL TEMPLATE FOR GENERATING MODELS BASED ON TRENDS
*** STRUCTURE:
*** 1. DEFINE LIST OF ALTERNATIVE MODELS - NAMES, REGRESSION COMMAND, PREDICT COMMAND
*** MODELS ARE NUMBERED 
*** 2. RUN THE MODELS DOWN THE LIST
*** - Generates extrapolation to the future
*** - Shows parameters projected to the future

*** Choose which models to run on a current run
global modelstorun "2005arima_A1M2"
global pcent "100" 
**** 1. PRELIMINARIES - MODELS, PARAMETERS

* Model1 : ARIMA with a 2-lag dep var - static
capture program drop reg_arimaL2s
program define reg_arimaL2s
	args modelname par i qminx qmaxx
	* Arima regression
	capture arima `par' L.`par' L2.`par' q $amlist if oca==`i' & q>=tq(`qminx') & q<=tq(`qmaxx')
	* Outreg
	capture outreg2 using simulations_$pcent/`modelname'_p/logfiles/`par'_`modelname'_reg, ctitle(`i') append
	* Predict the trend, including the past values (from qminx) - to see the model fit
	* The predicted parameters for the past have a suffix _tp 
	capture predict `par'_tp_`i' if oca==`i' & q>=tq(`qminx')
	* iii. 	Predict the trend, only to the future (from 2015q3)
	* The predicted parameters for the future have a suffix _tf 
	capture predict `par'_tf_`i' if oca==`i' & q>=tq(2015q3)
end

* Model2: ARIMA with a 2-lag dep var - dynamic forecast
capture program drop reg_arimaL2d
program define reg_arimaL2d
	args modelname par i qminx qmaxx
	capture arima `par' L.`par' L2.`par' q $amlist if oca==`i' & q>=tq(`qminx') & q<=tq(`qmaxx')
	capture outreg2 using simulations_$pcent/`modelname'_p/logfiles/`par'_`modelname'_reg, ctitle(`i') append
	capture predict `par'_tp_`i' if oca==`i' & q>=tq(`qminx'), dyn(q) t0(tq(`qminx'))
	capture predict `par'_tf_`i' if oca==`i', dyn(tq(2015q3))
end

* Model3: ARIMA with a 1-lag dep var - dynamic forecast
capture program drop reg_arimaL1d
program define reg_arimaL1d
	args modelname par i qminx qmaxx
	capture arima `par' L.`par' q $amlist if oca==`i' & q>=tq(`qminx') & q<=tq(`qmaxx')
	capture outreg2 using simulations_$pcent/`modelname'_p/logfiles/`par'_`modelname'_reg.txt, ctitle(`i') excel append
	capture predict `par'_tp_`i' if oca==`i' & q>=tq(`qminx'), dyn(q) t0(tq(`qminx'))
	capture predict `par'_tf_`i' if oca==`i', dyn(tq(2015q3))
end

* Model: ARIMA without a lag with, and a AR(1) and MA(1) error structure
capture program drop reg_2007arima_A1M1
program define reg_2007arima_A1M1
	args modelname par i qminx qmaxx
	capture arima `par' q $amlist if oca==`i' & q>=tq(`qminx') & q<=tq(`qmaxx'), ar(1) ma(1)
	capture outreg2 using simulations_$pcent/`modelname'_p/logfiles/`par'_`modelname'_reg, ctitle(`i') excel append
	capture predict `par'_tp_`i' if oca==`i' & q>=tq(`qminx'), dyn(q) t0(tq(`qminx'))
	capture predict `par'_tf_`i' if oca==`i', dyn(tq(2015q3))
end

* Model: ARIMA without a lag with and a AR(1 2) and MA(1) error structure
capture program drop reg_2007arima_A2M1
program define reg_2007arima_A2M1
	args modelname par i qminx qmaxx
	capture arima `par' q $amlist if oca==`i' & q>=tq(`qminx') & q<=tq(`qmaxx'),  ar(1 2) ma(1)
	capture outreg2 using simulations_$pcent/`modelname'_p/logfiles/`par'_`modelname'_reg.txt, ctitle(`i') excel append	
	capture predict `par'_tp_`i' if oca==`i' & q>=tq(`qminx'), dyn(q) t0(tq(`qminx'))
	capture predict `par'_tf_`i' if oca==`i', dyn(tq(2015q3))
end

* Model: ARIMA without a lag with and a AR(1 2 3) and MA(1) error structure
capture program drop reg_2007arima_A3M1
program define reg_2007arima_A3M1
	args modelname par i qminx qmaxx
	capture arima `par' q $amlist if oca==`i' & q>=tq(`qminx') & q<=tq(`qmaxx'),  ar(1 2 3) ma(1)
	capture outreg2 using simulations_$pcent/`modelname'_p/logfiles/`par'_`modelname'_reg.txt, ctitle(`i') excel append	
	capture predict `par'_tp_`i' if oca==`i' & q>=tq(`qminx'), dyn(q) t0(tq(`qminx'))
	capture predict `par'_tf_`i' if oca==`i', dyn(tq(2015q3))
end

* Model: ARIMA without a lag with and a AR(1) and MA(1 2) error structure
capture program drop reg_2005arima_A1M2
program define reg_2005arima_A1M2
	args modelname par i qminx qmaxx
	capture arima `par' q $amlist if oca==`i' & q>=tq(`qminx') & q<=tq(`qmaxx'),  ar(1) ma(1 2)
	capture outreg2 using simulations_$pcent/`modelname'_p/logfiles/`par'_`modelname'_reg.txt, ctitle(`i') excel append	
	capture predict `par'_tp_`i' if oca==`i' & q>=tq(`qminx'), dyn(q) t0(tq(`qminx'))
	capture predict `par'_tf_`i' if oca==`i', dyn(tq(2015q3))
end

* Model: ARIMA without a lag with and a AR(1 2) and MA(1 2) error structure
capture program drop reg_2007arima_A2M2
program define reg_2007arima_A2M2
	args modelname par i qminx qmaxx
	capture arima `par' q $amlist if oca==`i' & q>=tq(`qminx') & q<=tq(`qmaxx'),  ar(1 2) ma(1 2)
	capture outreg2 using simulations_$pcent/`modelname'_p/logfiles/`par'_`modelname'_reg.txt, ctitle(`i') excel append	
	capture predict `par'_tp_`i' if oca==`i' & q>=tq(`qminx'), dyn(q) t0(tq(`qminx'))
	capture predict `par'_tf_`i' if oca==`i', dyn(tq(2015q3))
end

* Model5: Prise-Winsten with a 2-lag dep var - static forecast only 
capture program drop reg_praisL2s
program define reg_praisL2s
	args modelname par i qminx qmaxx
	capture prais `par' L.`par' L2.`par' q $amlist if oca==`i' & q>=tq(`qminx') & q<=tq(`qmaxx'), corc
	capture outreg2 using simulations_$pcent/`modelname'_p/logfiles/`par'_`modelname'_reg.txt, ctitle(`i') append
	capture predict `par'_tp_`i' if oca==`i' & q>=tq(`qminx')
	capture predict `par'_tf_`i' if oca==`i' & q>=(tq(2015q3))
end

clear
set more off
set trace off
capture log close
* set the percent sample
global pcent "100"

* set the lists of parameters and the corresponding years over which we average
* parlist 1: most parameters, averaging over 2014
global parlist1 "z2 z3 p1 p2 p3 p4 pe_a3 pe_o p_a1 p_a2 la1 s_ls s_hs p4_hs pe_hsls p_z4 s_a1 s_a2 s_a3"
* parlist 2: because of data delay, averaging over 2013
global parlist2 "c1 z1"
* parlist 3: prison entry/release, already availabel in 2015, averaging over 2014q2-2015q1
global parlist3 "la3 vr_a1 vr_a2 vr_a3"
* parlist 4 - p5 is special, last usable value is 2013q2
global parlist4 "p5"
* parlist 4: technical parameters plus la2 -  rule of reason, keep them fixed as in the base model
global parlist5 "p12 p23 p45 la2"

**** 2. LOOP OVER MODELS
**** 2.1. GETTING THE DATA, ADJUSTING THE AMNESTY PARAMETERS 
**** MUST UPDATE THE NUMBERS, BUT MAY ALSO PICK THE NEW MODELS
foreach modelname of global modelstorun {
capture mkdir simulations_$pcent/`modelname'_p
capture mkdir simulations_$pcent/`modelname'_f
capture mkdir simulations_$pcent/`modelname'_p/logfiles

use simulations_$pcent/base/base_paramavg_$pcent
*keep oca q $parlist1 $parlist2 $parlist3  am_ls am_hs am_a1 am_a2 am_a3 ama1 ama2 ama3
save simulations_$pcent/`modelname'_p/`modelname'_p_paramavg_$pcent, replace
* Set as time series (if not done so yet)
tsset oca q, quarterly

*** The amnesty parameters are truncated in 2015q2, yet they are needed to converge to zero 
*** let them drop to zero by 2016q2
global amlist "amv_rp amv_a1 amv_a2 amv_a3"
foreach v of global amlist {
	replace `v' = 0 if q>=tq(2016q2)
	by oca: ipolate `v' q, g(`v'_hat)
	replace `v' = `v'_hat if q>=tq(2015q2)
	drop `v'_hat
}

**** 2.2. ESTIMATING THE TREND REGRESSION FOR A GIVEN MODEL AND GENERATE PREDICTED VALUES
* Loop over the three parameter lists
local qminlist "2005q1 2005q1 2005q1 2005q1"
local qmaxlist "2014q4 2013q4 2015q1 2013q2"
log using simulations_$pcent/`modelname'_p/logfiles/log_`modelname', replace
forvalues j = 1 (1) 4 {
	local qminx : word `j' of `qminlist'
	local qmaxx : word `j' of `qmaxlist'
	* loop over every parameters
	foreach par of global parlist`j' {
		capture erase simulations_$pcent/`modelname'_p/logfiles/`par'_`modelname'_reg.txt
		capture erase simulations_$pcent/`modelname'_p/logfiles/`par'_`modelname'_reg.xml
		* create matrix for filling in criterions
		matrix Crit = J(4,9,.)
		matrix rownames Crit = AIK BIC WALD Sig
		matrix colnames Crit = oca1 oca2 oca3 oca4 oca5 oca6 oca7 oca8 oca9
		* loop over offense categories		
		forvalues i = 1 (1) 9 {
			display "model = `modelname'"
			display "parameter = `par'"
				
			* i. Estimate the time series process and generate the prediction
			* that is, run the regression program defined up
			reg_`modelname' `modelname' `par' `i' `qminx' `qmaxx'
			* generate criterions
			cap quietly estat ic
			* fill them inside matrix
			matrix Te=r(S)
			* Aike
			matrix Crit[1,`i']=Te[1,5]
			* Schwartz/Bayes
			matrix Crit[2,`i']=Te[1,6]
			* Wald model significance (!not criteria!)
			matrix Crit[3,`i']=e(chi2)
			* Overal model significance
			matrix Crit[4,`i']=e(p)
			cap clear results
		}
		* print out matrix with criterions
		mat2txt2 Crit using simulations_$pcent/`modelname'_p/logfiles/`par'_`modelname'_crit.txt, replace tab label format(%4.1f) title(`par' with `modelname') 

		*ii. lump the predicted values for each oca into one variable
		g `par'_tf = .
		g `par'_tp = .
		forvalues i = 1 (1) 9 {
			capture replace `par'_tp = `par'_tp_`i' if oca == `i'
			capture replace `par'_tf = `par'_tf_`i' if oca == `i'
			capture drop `par'_tp_`i' `par'_tf_`i'
		}
		* iii. cap: do not allow the future values to deviate by more than 50% from the last year average
		* create variable equal to teh last year average for each oca, than do the cut
		bysort oca: egen `par'_lastmean = mean(`par') if q<=tq(`qmaxx') & q>tq(`qmaxx') - 4
		bysort oca: egen `par'_benchmark = max(`par'_lastmean)
		capture replace `par'_tp = `par'_benchmark*0.5 if q>=tq(2015q3) & `par'_tp<`par'_benchmark*0.5 & `par'_benchmark~=.
		capture replace `par'_tf = `par'_benchmark*0.5 if q>=tq(2015q3) & `par'_tf<`par'_benchmark*0.5 & `par'_benchmark~=.
		capture replace `par'_tp = `par'_benchmark*1.5 if q>=tq(2015q3) & `par'_tp>`par'_benchmark*1.5 & `par'_benchmark~=.
		capture replace `par'_tf = `par'_benchmark*1.5 if q>=tq(2015q3) & `par'_tf>`par'_benchmark*1.5 & `par'_benchmark~=.
		drop `par'_lastmean `par'_benchmark
	}
}
save simulations_$pcent/`modelname'_p/`modelname'_p_paramavg_$pcent, replace
log close
*/
*** 2.3. AFTER PREDICTION, ADJUST THE DATASETS SUCH THAT THEY HAVE THE SAME STRUCTURE AS BASE PARAMETERS FOR THE PRISON SIMULATION
* Correction of future probabilities and sentences
	use simulations_$pcent/`modelname'_p/`modelname'_p_paramavg_$pcent.dta

	local problist "p2 p3 p4 p5 pe_a3 pe_o p_a1 p_a2 vr_a1 vr_a2 vr_a3 p4_hs pe_hsls p_z4"
	local sentlist "s_hs s_ls s_a1 s_a2 s_a3"

	foreach v of local problist {
		replace `v'_tp = 0 if `v'_tp <0 & q>=tq(2015q3)
		replace `v'_tp = 1 if `v'_tp>1 & `v'_tp~=. & q>=tq(2015q3)
		replace `v'_tf = 0 if `v'_tf <0 & q>=tq(2015q3)
		replace `v'_tf = 1 if `v'_tf>1 & `v'_tf~=. & q>=tq(2015q3)
	}

	foreach v of local sentlist {
		replace `v'_tp = 0.1 if `v'_tp<0 & q>=tq(2015q3)
		replace `v'_tf = 0.1 if `v'_tf<0 & q>=tq(2015q3)
	}
* Correction for vra2 for murders - was equal to 0 all the time, 
* some regresions play havoc because of that
replace vr_a2_tp = 0 if oca==1
replace vr_a2_tf = 0 if oca==1

* Correction for la3 for driving - it should be equal to 8
* but whay no variation?
replace la3_tp = 8 if oca==5 & q>=tq(2001q1)
replace la3_tf = 8 if oca==5 & q>=tq(2001q1)

sort oca q
save simulations_$pcent/`modelname'_p/`modelname'_p_paramavg_$pcent, replace

**** Out of this, create two datasets (and two directories)
*** suffix _p, containing the parameter predictions for the past (and future)
*** suffix_f, containing the parameter predictions for the future)
*** In these files, the parameter are named such that the prediction (trend) has a pure name and the base is b_

* Save the _f file
save simulations_$pcent/`modelname'_f/`modelname'_f_paramavg_$pcent, replace
* Flip the predicted and the base parameters
* Must loop over all parameters
	forvalues j = 1 (1) 4 {
		foreach par of global parlist`j' {
			g b_`par' = `par'
			* Replace the base parameters with predicted ones for the future
			replace `par' = `par'_tf if q>=tq(2015q3)
			drop `par'_tf `par'_tp
		}
	}
* save the final _f file
sort oca q
save simulations_$pcent/`modelname'_f/`modelname'_f_paramavg_$pcent, replace

** Get back to the _p file and perform the same operation
use simulations_$pcent/`modelname'_p/`modelname'_p_paramavg_$pcent
forvalues j = 1 (1) 4 {
		foreach par of global parlist`j' {
			g b_`par' = `par'
			* Replace the base parameters with predicted ones whenever the differ
			replace `par' = `par'_tp if `par'~=`par'_tp & `par'_tp~=.
			drop `par'_tf `par'_tp
		}
	}
* save the final _p file
sort oca q
save simulations_$pcent/`modelname'_p/`modelname'_p_paramavg_$pcent, replace


*** 2.4. SIMULATE THE PRISON POPULATIONS - MODEL WITH FUTURE PARAMETERS

**** Simulate the prison population - projecting to the future (`modelname')
do dopris3_modelavg	
pminflowsimul `modelname'_f

*** Simulate the prison population - using the predicted past parameters
*** (useful to see the effects of some wrongly predicted parameters)
pminflowsimul `modelname'_p

*** Compare the projection to the future with base
do dopris4_descriptive
modelcompare `modelname'_f base

*** Compare the projection with the past projected parameter with base 
* this is mainly needed to see the fit of the predicted  parameters 
modelcompare `modelname'_p base

}


